package org.jeecg.modules.ctop.controller;


import cn.com.ctop.common.module.entity.MaterialInfo;
import cn.com.ctop.common.module.service.IMaterialAscriptionService;
import cn.com.ctop.common.module.service.IMaterialInfoService;
import cn.com.ctop.common.module.service.IMaterialParameterService;
import cn.com.ctop.common.module.service.IMaterialTagService;
import cn.com.ctop.common.module.utils.Check;
import cn.com.ctop.common.module.utils.KuaishouInterfaceConstant;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.shiro.SecurityUtils;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.aspect.annotation.AutoLog;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.common.system.vo.LoginUser;
import org.jeecg.common.util.DateUtils;
import org.jeecg.common.util.oConvertUtils;
import org.jeecg.modules.ctop.service.IProjectMemberService;
import org.jeecgframework.poi.excel.ExcelImportUtil;
import org.jeecgframework.poi.excel.def.NormalExcelConstants;
import org.jeecgframework.poi.excel.entity.ExportParams;
import org.jeecgframework.poi.excel.entity.ImportParams;
import org.jeecgframework.poi.excel.view.JeecgEntityExcelView;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.text.ParseException;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

/**
 * 素材信息
 *
 * @author jeecg-boot
 * @version V1.0
 * @date 2019-10-11
 */
@Slf4j
@Api(tags = "素材信息")
@RestController
@RequestMapping("/ctop/materialInfo")
public class MaterialInfoController {
    @Autowired
    private IMaterialInfoService materialInfoService;
    @Autowired
    private IMaterialAscriptionService materialAscriptionService;
    @Autowired
    private IMaterialParameterService materialParameterService;
    @Autowired
    private IMaterialTagService materialTagService;

    @GetMapping(value = "/report")
    public JSONObject report(String userId) {
        return materialInfoService.report(userId);
    }


    @GetMapping(value = "/getDetail")
    public JSONObject getDetail(String materialId) {
        return materialInfoService.getDetail(materialId);

    }

    @PostMapping("watermark")
    public Result<Object> watermarkVideo(@RequestParam(name = "id") String id, @RequestParam(name = "watermarkTemplateId") Integer watermarkTemplateId) {
        Result<Object> result = new Result<>();
        LoginUser sysUser = (LoginUser) SecurityUtils.getSubject().getPrincipal();
        materialInfoService.watermarkVideo(id, watermarkTemplateId, sysUser.getId());
        result.setCode(200);
        result.setSuccess(true);
        result.setMessage("水印合成任务执行成功，请稍后查看");
        return result;
    }

    @GetMapping("excellent/set")
    public Result<Object> setExcellent(@RequestParam(name = "id") String code, @RequestParam(name = "excellent") Integer excellent) {
        Result<Object> result = new Result<>();
        materialInfoService.setExcellent(code, excellent);
        result.setCode(200);
        result.setSuccess(true);
        result.setMessage("设置成功");
        return result;
    }


    /**
     * 分页列表查询
     *
     * @param materialInfo
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    @Autowired
    private IProjectMemberService memberService;

    @GetMapping("/excellent/list")
    public Result<IPage<MaterialInfo>> excellentList(MaterialInfo materialInfo,
                                                     @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                     @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                                     HttpServletRequest req) {
        Result<IPage<MaterialInfo>> result = new Result<>();
        QueryWrapper<MaterialInfo> queryWrapper = new QueryWrapper<>();
        Object createTime = materialInfo.getCreateTime();
        materialInfo.setCreateTime(null);
        materialInfo.setExcellent(1);
        queryWrapper = QueryGenerator.initQueryWrapper(materialInfo, req.getParameterMap());
        if (!Check.isNull(createTime)) {
            try {
                Map<String, Object> map = DateUtils.getStartEndTime(createTime.toString());
                queryWrapper.ge("create_time", map.get("start"));
                queryWrapper.lt("create_time", map.get("end"));
            } catch (ParseException e) {
                e.printStackTrace();
            }
        }

        Page<MaterialInfo> page = new Page<>(pageNo, pageSize);
        IPage<MaterialInfo> pageList = materialInfoService.page(page, queryWrapper);
        result.setSuccess(true);
        result.setResult(pageList);
        return result;
    }

    @AutoLog(value = "素材信息-分页列表查询")
    @ApiOperation(value = "素材信息-分页列表查询", notes = "素材信息-分页列表查询")
    @GetMapping(value = "/list")
    public Result<IPage<MaterialInfo>> queryPageList(MaterialInfo materialInfo,
                                                     @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                     @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                                     HttpServletRequest req) {

        Result<IPage<MaterialInfo>> result = new Result<>();
        QueryWrapper<MaterialInfo> queryWrapper = new QueryWrapper<>();
        Object createTime = materialInfo.getCreateTime();
        materialInfo.setCreateTime(null);
        if (!Check.isNull(materialInfo.getUserId())) {
            String roleCode = materialInfoService.getRoleCodeByUserId(materialInfo.getUserId());
            if ("admin".equals(roleCode) || "operator".equals(roleCode) || "kuaishouOperationManager".equals(roleCode) || "operationAssistant".equals(roleCode) || "touTiaoOperationManager".equals(roleCode)) {
                materialInfo.setUserId(null);
                queryWrapper = QueryGenerator.initQueryWrapper(materialInfo, req.getParameterMap());
            } else if ("designTeamLeader".equals(roleCode) || "planeLeader".equals(roleCode) || "plane".equals(roleCode) || "plan".equals(roleCode) || "shot".equals(roleCode)) {
                String userId = materialInfo.getUserId();
                materialInfo.setUserId(null);
                queryWrapper = QueryGenerator.initQueryWrapper(materialInfo, req.getParameterMap());
                if (Check.isNull(materialInfo.getProjectId())) {
                    List<Long> projects = memberService.selectProjectsByUserId(userId);
                    if (!Check.isNull(projects)) {
                        queryWrapper.in("project_id", projects);
                    } else {
                        result.setSuccess(true);
                        result.setResult(null);
                        return result;
                    }
                }

            } else {
                queryWrapper = QueryGenerator.initQueryWrapper(materialInfo, req.getParameterMap());
            }
            if (!Check.isNull(createTime)) {
                try {
                    Map<String, Object> map = DateUtils.getStartEndTime(createTime.toString());
                    queryWrapper.ge("create_time", map.get("start"));
                    queryWrapper.lt("create_time", map.get("end"));
                } catch (ParseException e) {
                    e.printStackTrace();
                }
            }
        }

        Page<MaterialInfo> page = new Page<>(pageNo, pageSize);
        IPage<MaterialInfo> pageList = materialInfoService.page(page, queryWrapper);
        result.setSuccess(true);
        result.setResult(pageList);
        return result;
    }

    /**
     * 添加
     *
     * @param materialInfo
     * @return
     */
    @AutoLog(value = "素材信息-添加")
    @ApiOperation(value = "素材信息-添加", notes = "素材信息-添加")
    @PostMapping(value = "/add")
    public Result<MaterialInfo> add(@RequestBody MaterialInfo materialInfo) {
        Result<MaterialInfo> result = new Result<>();
        try {
            materialInfoService.save(materialInfo);
            result.success("添加成功！");
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            result.error500("操作失败");
        }
        return result;
    }


    /**
     * 批量修改项目
     *
     * @param jsonObject
     * @return
     */
    @PostMapping(value = "/batchEdit")
    public Result<MaterialInfo> batchEdit(@RequestBody JSONObject jsonObject) {
        Result<MaterialInfo> result = new Result<>();
        try {
            Long projectId = jsonObject.getLong("projectId");
            JSONArray materialIds = jsonObject.getJSONArray("materialIds");
            for (int i = 0; i < materialIds.size(); i++) {
                String materialId = materialIds.getString(i);
                MaterialInfo updateMaterial = new MaterialInfo();
                updateMaterial.setId(materialId);
                updateMaterial.setProjectId(projectId);
                materialInfoService.updateById(updateMaterial);

            }
            result.setSuccess(true);
            result.success("修改成功!");
        } catch (Exception e) {
            result.setSuccess(false);
        }


        return result;

    }


    /**
     * 编辑
     *
     * @param materialInfo
     * @return
     */
    @AutoLog(value = "素材信息-编辑")
    @ApiOperation(value = "素材信息-编辑", notes = "素材信息-编辑")
    @PutMapping(value = "/edit")
    public Result<MaterialInfo> edit(@RequestBody MaterialInfo materialInfo) {
        Result<MaterialInfo> result = new Result<>();
        MaterialInfo materialInfoEntity = materialInfoService.getById(materialInfo.getId());
        if (materialInfoEntity == null) {
            result.error500("未找到对应实体");
        } else {
            String url = materialInfo.getUrl();
            if (!Check.isNull(url)) {
                if (!url.contains(KuaishouInterfaceConstant.HTTPS_PREFIX)) {
                    url = KuaishouInterfaceConstant.HTTPS_PREFIX + url;
                }
                materialInfo.setUrl(url);
            }

            boolean ok = materialInfoService.updateById(materialInfo);
            if (ok) {
                result.success("修改成功!");
            }
        }

        return result;

    }

    /**
     * 通过id删除
     *
     * @param id
     * @return
     */
    @AutoLog(value = "素材信息-通过id删除")
    @ApiOperation(value = "素材信息-通过id删除", notes = "素材信息-通过id删除")
    @DeleteMapping(value = "/delete")
    public Result<?> delete(@RequestParam(name = "id", required = true) String id) {
        try {
            materialInfoService.removeById(id);
            Map<String, Object> deleteMap = new HashMap<>();
            deleteMap.put("material_id", id);

            materialAscriptionService.removeByMap(deleteMap);
            materialParameterService.removeByMap(deleteMap);
            materialTagService.removeByMap(deleteMap);
        } catch (Exception e) {
            log.error("删除失败", e.getMessage());
            return Result.error("删除失败!");
        }
        return Result.ok("删除成功!");
    }

    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @AutoLog(value = "素材信息-批量删除")
    @ApiOperation(value = "素材信息-批量删除", notes = "素材信息-批量删除")
    @DeleteMapping(value = "/deleteBatch")
    public Result<MaterialInfo> deleteBatch(@RequestParam(name = "ids", required = true) String ids) {
        Result<MaterialInfo> result = new Result<>();
        if (ids == null || "".equals(ids.trim())) {
            result.error500("参数不识别！");
        } else {
            this.materialInfoService.removeByIds(Arrays.asList(ids.split(",")));
            result.success("删除成功!");
        }
        return result;
    }

    /**
     * 通过id查询
     *
     * @param id
     * @return
     */
    @AutoLog(value = "素材信息-通过id查询")
    @ApiOperation(value = "素材信息-通过id查询", notes = "素材信息-通过id查询")
    @GetMapping(value = "/queryById")
    public Result<MaterialInfo> queryById(@RequestParam(name = "id", required = true) String id) {
        Result<MaterialInfo> result = new Result<MaterialInfo>();
        MaterialInfo materialInfo = materialInfoService.getById(id);
        if (materialInfo == null) {
            result.error500("未找到对应实体");
        } else {
            result.setResult(materialInfo);
            result.setSuccess(true);
        }
        return result;
    }

    /**
     * 导出excel
     *
     * @param request
     * @param response
     */
    @RequestMapping(value = "/exportXls")
    public ModelAndView exportXls(HttpServletRequest request, HttpServletResponse response) {
        // Step.1 组装查询条件
        QueryWrapper<MaterialInfo> queryWrapper = null;
        try {
            String paramsStr = request.getParameter("paramsStr");
            if (oConvertUtils.isNotEmpty(paramsStr)) {
                String deString = URLDecoder.decode(paramsStr, "UTF-8");
                MaterialInfo materialInfo = JSON.parseObject(deString, MaterialInfo.class);
                queryWrapper = QueryGenerator.initQueryWrapper(materialInfo, request.getParameterMap());
            }
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }

        //Step.2 AutoPoi 导出Excel
        ModelAndView mv = new ModelAndView(new JeecgEntityExcelView());
        List<MaterialInfo> pageList = materialInfoService.list(queryWrapper);
        //导出文件名称
        mv.addObject(NormalExcelConstants.FILE_NAME, "素材信息列表");
        mv.addObject(NormalExcelConstants.CLASS, MaterialInfo.class);
        mv.addObject(NormalExcelConstants.PARAMS, new ExportParams("素材信息列表数据", "导出人:Jeecg", "导出信息"));
        mv.addObject(NormalExcelConstants.DATA_LIST, pageList);
        return mv;
    }


    /**
     * 通过excel导入数据
     *
     * @param request
     * @param response
     * @return
     */
    @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
    public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
        MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
        Map<String, MultipartFile> fileMap = multipartRequest.getFileMap();
        for (Map.Entry<String, MultipartFile> entity : fileMap.entrySet()) {
            MultipartFile file = entity.getValue();
            ImportParams params = new ImportParams();
            params.setTitleRows(2);
            params.setHeadRows(1);
            params.setNeedSave(true);
            try {
                List<MaterialInfo> listMaterialInfos = ExcelImportUtil.importExcel(file.getInputStream(), MaterialInfo.class, params);
                materialInfoService.saveBatch(listMaterialInfos);
                return Result.ok("文件导入成功！数据行数:" + listMaterialInfos.size());
            } catch (Exception e) {
                log.error(e.getMessage(), e);
                return Result.error("文件导入失败:" + e.getMessage());
            } finally {
                try {
                    file.getInputStream().close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
        return Result.ok("文件导入失败！");
    }

}
