package org.jeecg.modules.ctop.controller;


import cn.com.ctop.bytedance.mapper.MaterialReportMapper;
import cn.com.ctop.common.module.utils.Check;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.shiro.SecurityUtils;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.aspect.annotation.AutoLog;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.common.system.vo.LoginUser;
import org.jeecg.common.util.oConvertUtils;
import org.jeecg.modules.ctop.entity.Advertiser;
import org.jeecg.modules.ctop.entity.Project;
import org.jeecg.modules.ctop.entity.ProjectMember;
import org.jeecg.modules.ctop.mapper.ProjectMapper;
import org.jeecg.modules.ctop.mapper.ProjectMemberMapper;
import org.jeecg.modules.ctop.service.IAdvertiserService;
import org.jeecg.modules.ctop.service.IProjectMemberService;
import org.jeecg.modules.ctop.service.IProjectService;
import org.jeecg.modules.system.entity.SysUser;
import org.jeecg.modules.system.mapper.UserCompanyMapper;
import org.jeecg.modules.system.service.ISysUserService;
import org.jeecgframework.poi.excel.ExcelImportUtil;
import org.jeecgframework.poi.excel.def.NormalExcelConstants;
import org.jeecgframework.poi.excel.entity.ExportParams;
import org.jeecgframework.poi.excel.entity.ImportParams;
import org.jeecgframework.poi.excel.view.JeecgEntityExcelView;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.util.*;

/**
 * 项目
 *
 * @author jeecg-boot
 * @version V1.0
 * @date 2019-11-19
 */
@Slf4j
@Api(tags = "项目")
@RestController
@RequestMapping("/ctop/project")
public class ProjectController {
    @Autowired
    private IProjectService projectService;
    @Autowired
    private IProjectMemberService projectMemberService;
    @Autowired
    private ProjectMemberMapper projectMemberMapper;
    @Autowired
    private IAdvertiserService advertiserService;
    @Autowired
    private ProjectMapper projectMapper;
    @Autowired
    private MaterialReportMapper materialReportMapper;
    @Autowired
    private UserCompanyMapper userCompanyMapper;

    /**
     * 根据 项目 媒体类型 查看项目
     *
     * @param advertiserId
     * @param mediaId
     * @return
     */

    @GetMapping(value = "/getProjectByAdvertiserAndMediaId")
    public Result<List<Project>> getProjectByCompany(String advertiserId, String mediaId) {
        Result<List<Project>> result = new Result<>();
        try {
            QueryWrapper<Project> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("media_id", mediaId);
            queryWrapper.eq("advertiser_id", advertiserId);
            List<Project> projects = projectMapper.selectList(queryWrapper);
            result.setSuccess(true);
            result.setResult(projects);
        } catch (Exception e) {
            e.printStackTrace();
            result.setSuccess(false);

        }
        return result;
    }


    /**
     * 我管理的
     *
     * @param userId
     * @return
     */
    @AutoLog(value = "项目成员-分页列表查询")
    @ApiOperation(value = "项目成员-分页列表查询", notes = "项目成员-分页列表查询")
    @GetMapping(value = "/responsibleList")
    public Map<String, Object> responsibleList(String userId) {
        Map<String, Object> map = new HashMap<>();
        String roleCode = materialReportMapper.getRoleCodeByUserId(userId);
        QueryWrapper<Project> queryWrapper = new QueryWrapper<>();
        List<Project> projects = new ArrayList<>();
        if (!"admin".equals(roleCode)) {
            if ("designTeamLeader".equals(roleCode) || "touTiaoOperationManager".equals(roleCode) || "kuaishouOperationManager".equals(roleCode) || "planeLeader".equals(roleCode)) {
                QueryWrapper<ProjectMember> memberQueryWrapper = new QueryWrapper<>();
                memberQueryWrapper.eq("user_id", userId);
                memberQueryWrapper.groupBy("project_id");
                memberQueryWrapper.orderByDesc("create_time");
                List<ProjectMember> projectMemberList = projectMemberMapper.selectList(memberQueryWrapper);
                if (!Check.isNull(projectMemberList)) {
                    for (ProjectMember member : projectMemberList) {
                        Long projectId = member.getProjectId();
                        Project project = projectService.getById(projectId);
                        projects.add(project);
                    }
                }
            } else {
                QueryWrapper<Project> projectQueryWrapper = new QueryWrapper<>();
                projectQueryWrapper.eq("responsible_id", userId);
                projectQueryWrapper.orderByDesc("create_time");
                List<Project> responsibles = projectMapper.selectList(projectQueryWrapper);
                if (!Check.isNull(responsibles)) {
                    projects = responsibles;
                } else {
                    QueryWrapper<Project> designResponsibleQueryWrapper = new QueryWrapper<>();
                    designResponsibleQueryWrapper.eq("design_responsible_id", userId);
                    designResponsibleQueryWrapper.orderByDesc("create_time");
                    List<Project> designResponsibles = projectMapper.selectList(projectQueryWrapper);
                    projects = designResponsibles;
                }


            }
        } else {
            queryWrapper.orderByDesc("create_time");
            projects = projectMapper.selectList(queryWrapper);
        }

        map.put("success", true);
        map.put("code", 0);
        map.put("result", projects);
        return map;
    }


    /**
     * 分页列表查询
     *
     * @param project
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    @AutoLog(value = "项目-分页列表查询")
    @ApiOperation(value = "项目-分页列表查询", notes = "项目-分页列表查询")
    @GetMapping(value = "/list")
    public Result<IPage<Project>> queryPageList(Project project,
                                                @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                                HttpServletRequest req) {
        Result<IPage<Project>> result = new Result<IPage<Project>>();
        QueryWrapper<Project> queryWrapper = QueryGenerator.initQueryWrapper(project, req.getParameterMap());
        Page<Project> page = new Page<Project>(pageNo, pageSize);

        LoginUser sysUser = (LoginUser) SecurityUtils.getSubject().getPrincipal();
        if (Check.isNull(sysUser)) {
            result.setSuccess(false);
            return result;
        }

        String roleCode = materialReportMapper.getRoleCodeByUserId(sysUser.getId());
        if (!"admin".equals(roleCode)) {
            queryWrapper.eq("user_id", sysUser.getId());
            queryWrapper.orderByDesc("create_time");
        }
        IPage<Project> pageList = projectService.page(page, queryWrapper);
        result.setSuccess(true);
        result.setResult(pageList);
        return result;
    }

    /**
     * 添加
     *
     * @param project
     * @return
     */
    @Autowired
    private ISysUserService sysUserService;

    @AutoLog(value = "项目-添加")
    @ApiOperation(value = "项目-添加", notes = "项目-添加")
    @PostMapping(value = "/add")
    public Result<Project> add(@RequestBody Project project) {
        Result<Project> result = new Result<Project>();
        try {
            String advertiserId = project.getAdvertiserId();
            Advertiser advertiser = advertiserService.getById(advertiserId);
            Map<String, String> memberMap = new HashMap<>();
            if (!Check.isNull(advertiser)) {
                project.setCompanyId(advertiser.getCompanyId());

            }
            SysUser sysUser = sysUserService.getById(project.getResponsibleId());
            if (!Check.isNull(sysUser)) {
                project.setResponsibleName(sysUser.getRealname());
                memberMap.put(sysUser.getId(), sysUser.getRealname());
            }


            SysUser designSysUser = sysUserService.getById(project.getDesignResponsibleId());
            if (!Check.isNull(designSysUser)) {
                project.setDesignResponsibleName(designSysUser.getRealname());
                memberMap.put(designSysUser.getId(), designSysUser.getRealname());
            }

            projectService.save(project);

            Long id = project.getId();
            for (Map.Entry<String, String> projectMember : memberMap.entrySet()) {

                String userId = projectMember.getKey();
                String userName = projectMember.getValue();
                Map<String, Object> map = new HashMap<>();
                map.put("project_id", id);
                map.put("user_id", userId);
                projectMemberService.removeByMap(map);
                ProjectMember member = new ProjectMember();
                member.setProjectId(id);
                member.setProjectName(project.getProjectName());
                member.setUserId(userId);
                member.setUserName(userName);
                String roleCode = materialReportMapper.getRoleCodeByUserId(userId);
                if (!Check.isNull(roleCode)) {
                    member.setRoleCode(roleCode);
                }
                projectMemberService.save(member);
            }


            result.success("添加成功！");
        } catch (Exception e) {
            e.printStackTrace();
            result.error500("操作失败");
        }
        return result;
    }

    /**
     * 编辑
     *
     * @param project
     * @return
     */
    @AutoLog(value = "项目-编辑")
    @ApiOperation(value = "项目-编辑", notes = "项目-编辑")
    @PutMapping(value = "/edit")
    public Result<Project> edit(@RequestBody Project project) {
        Result<Project> result = new Result<Project>();

        SysUser sysUser = sysUserService.getById(project.getResponsibleId());
        if (!Check.isNull(sysUser)) {
            project.setResponsibleName(sysUser.getRealname());
            projectMemberService.addMember(sysUser, project);
        }

        SysUser designSysUser = sysUserService.getById(project.getDesignResponsibleId());
        if (!Check.isNull(designSysUser)) {
            project.setDesignResponsibleName(designSysUser.getRealname());
            projectMemberService.addMember(designSysUser, project);
        }
        Project projectEntity = projectService.getById(project.getId());
        if (projectEntity == null) {
            result.error500("未找到对应实体");
        } else {
            boolean ok = projectService.updateById(project);


            QueryWrapper<ProjectMember> memberQueryWrapper = new QueryWrapper<>();
            memberQueryWrapper.eq("project_id", project.getId());
            ProjectMember updateMember = new ProjectMember();
            updateMember.setProjectName(project.getProjectName());
            projectMemberService.update(updateMember, memberQueryWrapper);
            //      projectMemberMapper.update()

            if (ok) {
                result.success("修改成功!");
            }
        }

        return result;
    }

    /**
     * 通过id删除
     *
     * @param id
     * @return
     */
    @AutoLog(value = "项目-通过id删除")
    @ApiOperation(value = "项目-通过id删除", notes = "项目-通过id删除")
    @DeleteMapping(value = "/delete")
    public Result<?> delete(@RequestParam(name = "id", required = true) String id) {
        try {
            projectService.removeById(id);
            Map<String, Object> deleteMap = new HashMap<>();
            deleteMap.put("project_id", id);
            projectMemberService.removeByMap(deleteMap);
        } catch (Exception e) {
            log.error("删除失败", e.getMessage());
            return Result.error("删除失败!");
        }
        return Result.ok("删除成功!");
    }

    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @AutoLog(value = "项目-批量删除")
    @ApiOperation(value = "项目-批量删除", notes = "项目-批量删除")
    @DeleteMapping(value = "/deleteBatch")
    public Result<Project> deleteBatch(@RequestParam(name = "ids", required = true) String ids) {
        Result<Project> result = new Result<Project>();
        if (ids == null || "".equals(ids.trim())) {
            result.error500("参数不识别！");
        } else {
            this.projectService.removeByIds(Arrays.asList(ids.split(",")));
            result.success("删除成功!");
        }
        return result;
    }

    /**
     * 通过id查询
     *
     * @param id
     * @return
     */
    @AutoLog(value = "项目-通过id查询")
    @ApiOperation(value = "项目-通过id查询", notes = "项目-通过id查询")
    @GetMapping(value = "/queryById")
    public Result<Project> queryById(@RequestParam(name = "id", required = true) String id) {
        Result<Project> result = new Result<Project>();
        Project project = projectService.getById(id);
        if (project == null) {
            result.error500("未找到对应实体");
        } else {
            result.setResult(project);
            result.setSuccess(true);
        }
        return result;
    }

    /**
     * 导出excel
     *
     * @param request
     * @param response
     */
    @RequestMapping(value = "/exportXls")
    public ModelAndView exportXls(HttpServletRequest request, HttpServletResponse response) {
        // Step.1 组装查询条件
        QueryWrapper<Project> queryWrapper = null;
        try {
            String paramsStr = request.getParameter("paramsStr");
            if (oConvertUtils.isNotEmpty(paramsStr)) {
                String deString = URLDecoder.decode(paramsStr, "UTF-8");
                Project project = JSON.parseObject(deString, Project.class);
                queryWrapper = QueryGenerator.initQueryWrapper(project, request.getParameterMap());
            }
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }

        //Step.2 AutoPoi 导出Excel
        ModelAndView mv = new ModelAndView(new JeecgEntityExcelView());
        List<Project> pageList = projectService.list(queryWrapper);
        //导出文件名称
        mv.addObject(NormalExcelConstants.FILE_NAME, "项目列表");
        mv.addObject(NormalExcelConstants.CLASS, Project.class);
        mv.addObject(NormalExcelConstants.PARAMS, new ExportParams("项目列表数据", "导出人:Jeecg", "导出信息"));
        mv.addObject(NormalExcelConstants.DATA_LIST, pageList);
        return mv;
    }

    /**
     * 通过excel导入数据
     *
     * @param request
     * @param response
     * @return
     */
    @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
    public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
        MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
        Map<String, MultipartFile> fileMap = multipartRequest.getFileMap();
        for (Map.Entry<String, MultipartFile> entity : fileMap.entrySet()) {
            MultipartFile file = entity.getValue();
            ImportParams params = new ImportParams();
            params.setTitleRows(2);
            params.setHeadRows(1);
            params.setNeedSave(true);
            try {
                List<Project> listProjects = ExcelImportUtil.importExcel(file.getInputStream(), Project.class, params);
                projectService.saveBatch(listProjects);
                return Result.ok("文件导入成功！数据行数:" + listProjects.size());
            } catch (Exception e) {
                log.error(e.getMessage(), e);
                return Result.error("文件导入失败:" + e.getMessage());
            } finally {
                try {
                    file.getInputStream().close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
        return Result.ok("文件导入失败！");
    }

}
