package org.jeecg.modules.ctop.job;

import cn.com.ctop.common.module.entity.CtopOauthToken;
import cn.com.ctop.common.module.service.ICtopOauthTokenService;
import cn.com.ctop.common.module.utils.CtopAdConstant;
import cn.com.ctop.kuaishou.modules.batch.service.IKuaishouInterfaceService;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.util.DateUtils;
import org.quartz.Job;
import org.quartz.JobExecutionContext;
import org.quartz.JobExecutionException;
import org.springframework.beans.factory.annotation.Autowired;

import java.util.Date;
import java.util.List;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

/**
 * 获取创意分天报表
 *
 * @author syh
 */
@Slf4j
public class KuaishouDailyCreativeReportLoadJob implements Job {
    @Autowired
    private ICtopOauthTokenService tokenService;
    @Autowired
    private IKuaishouInterfaceService kuaishouInterfaceService;

    static ExecutorService executorService = null;

    @Override
    public void execute(JobExecutionContext jobExecutionContext) throws JobExecutionException {
        Thread thread = new Thread() {
            @Override
            public void run() {
                Date deleteDate = DateUtils.addDay(new Date(), -2);
                kuaishouInterfaceService.deleteHourlyReport(deleteDate);
                Date getDate = DateUtils.addDay(new Date(), -1);
                //1:查询当日数据
                executorService = Executors.newFixedThreadPool(3);
                List<CtopOauthToken> tokens = tokenService.getTokenListByType(CtopAdConstant.PLATFORM_TYPE_KUAISHOU);
                if (null == tokens || tokens.size() <= 0) {
                    log.info("定时获取快手数据异常：未获取到可用的token");
                    return;
                }
                log.info("开始分天创意报表拉去完成");
                for (CtopOauthToken token : tokens) {
                    executorService.submit(new Runnable() {
                        @Override
                        public void run() {
                            log.info("获取创意分天报表开始,accountId:{}", token.getAccountId());
                            kuaishouInterfaceService.getAdvertiserCreativeReportDaily(token, getDate, getDate);
                        }
                    });


                }
            }
        };
        thread.start();

    }
}
