package cn.com.ctop.toutiao.controller;

import cn.com.ctop.common.module.utils.StringUtils;
import cn.com.ctop.toutiao.entity.BytedanceAppDownloadDetailPage;
import cn.com.ctop.toutiao.service.IBytedanceAppDownloadDetailPageService;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.aspect.annotation.AutoLog;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.common.util.oConvertUtils;
import org.jeecgframework.poi.excel.ExcelImportUtil;
import org.jeecgframework.poi.excel.def.NormalExcelConstants;
import org.jeecgframework.poi.excel.entity.ExportParams;
import org.jeecgframework.poi.excel.entity.ImportParams;
import org.jeecgframework.poi.excel.view.JeecgEntityExcelView;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.util.Arrays;
import java.util.List;
import java.util.Map;

/**
 * 应用下载详情页信息
 *
 * @author jeecg-boot
 * @version V1.0
 * @date 2019-09-27
 */
@Slf4j
@Api(tags = "应用下载详情页信息")
@RestController
@RequestMapping("/ctop/bytedanceAppDownloadDetailPage")
public class BytedanceAppDownloadDetailPageController {
    @Autowired
    private IBytedanceAppDownloadDetailPageService bytedanceAppDownloadDetailPageService;

    /**
     * 分页列表查询
     *
     * @param bytedanceAppDownloadDetailPage
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    @AutoLog(value = "应用下载详情页信息-分页列表查询")
    @ApiOperation(value = "应用下载详情页信息-分页列表查询", notes = "应用下载详情页信息-分页列表查询")
    @GetMapping(value = "/list")
    public Result<IPage<BytedanceAppDownloadDetailPage>> queryPageList(BytedanceAppDownloadDetailPage bytedanceAppDownloadDetailPage,
                                                                       @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                                       @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                                                       HttpServletRequest req) {
        Result<IPage<BytedanceAppDownloadDetailPage>> result = new Result<>();
        QueryWrapper<BytedanceAppDownloadDetailPage> queryWrapper = QueryGenerator.initQueryWrapper(bytedanceAppDownloadDetailPage, req.getParameterMap());
        Page<BytedanceAppDownloadDetailPage> page = new Page<>(pageNo, pageSize);
        IPage<BytedanceAppDownloadDetailPage> pageList = bytedanceAppDownloadDetailPageService.page(page, queryWrapper);
        result.setSuccess(true);
        result.setResult(pageList);
        return result;
    }

    /**
     * 添加
     *
     * @param appDownloadDetailPage
     * @return
     */
    @AutoLog(value = "应用下载详情页信息-添加")
    @ApiOperation(value = "应用下载详情页信息-添加", notes = "应用下载详情页信息-添加")
    @PostMapping(value = "/add")
    public Result<BytedanceAppDownloadDetailPage> add(@RequestBody BytedanceAppDownloadDetailPage appDownloadDetailPage) {
        Result<BytedanceAppDownloadDetailPage> result = new Result<>();
        try {
            if (null == appDownloadDetailPage.getName() || "".equals(appDownloadDetailPage.getName().trim()) || null == appDownloadDetailPage.getDownloadUrl() || "".equals(appDownloadDetailPage.getDownloadUrl().trim()) || null == appDownloadDetailPage.getAppName() || "".equals(appDownloadDetailPage.getAppName().trim())) {
                result.error500("参数异常");
                return result;
            }
            QueryWrapper<BytedanceAppDownloadDetailPage> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("name", appDownloadDetailPage.getName()).last("limit 1");
            BytedanceAppDownloadDetailPage detailPage = bytedanceAppDownloadDetailPageService.getOne(queryWrapper);
            if (null != detailPage) {
                result.error500("该名称已存在");
                return result;
            }
            bytedanceAppDownloadDetailPageService.save(appDownloadDetailPage);
            result.success("添加成功！");
            return result;
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            result.error500("操作失败");
            return result;
        }
    }

    /**
     * 编辑
     *
     * @param bytedanceAppDownloadDetailPage
     * @return
     */
    @AutoLog(value = "应用下载详情页信息-编辑")
    @ApiOperation(value = "应用下载详情页信息-编辑", notes = "应用下载详情页信息-编辑")
    @PutMapping(value = "/edit")
    public Result<BytedanceAppDownloadDetailPage> edit(@RequestBody BytedanceAppDownloadDetailPage bytedanceAppDownloadDetailPage) {
        Result<BytedanceAppDownloadDetailPage> result = new Result<>();
        BytedanceAppDownloadDetailPage bytedanceAppDownloadDetailPageEntity = bytedanceAppDownloadDetailPageService.getById(bytedanceAppDownloadDetailPage.getId());
        if (bytedanceAppDownloadDetailPageEntity == null) {
            result.error500("未找到对应实体");
        } else {
            boolean ok = bytedanceAppDownloadDetailPageService.updateById(bytedanceAppDownloadDetailPage);
            if (ok) {
                result.success("修改成功!");
            }
        }

        return result;
    }

    /**
     * 通过id删除
     *
     * @param id
     * @return
     */
    @AutoLog(value = "应用下载详情页信息-通过id删除")
    @ApiOperation(value = "应用下载详情页信息-通过id删除", notes = "应用下载详情页信息-通过id删除")
    @DeleteMapping(value = "/delete")
    public Result<?> delete(@RequestParam(name = "id", required = true) String id) {
        try {
            bytedanceAppDownloadDetailPageService.removeById(id);
        } catch (Exception e) {
            log.error("删除失败", e.getMessage());
            return Result.error("删除失败!");
        }
        return Result.ok("删除成功!");
    }

    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @AutoLog(value = "应用下载详情页信息-批量删除")
    @ApiOperation(value = "应用下载详情页信息-批量删除", notes = "应用下载详情页信息-批量删除")
    @DeleteMapping(value = "/deleteBatch")
    public Result<BytedanceAppDownloadDetailPage> deleteBatch(@RequestParam(name = "ids", required = true) String ids) {
        Result<BytedanceAppDownloadDetailPage> result = new Result<>();
        if (ids == null || "".equals(ids.trim())) {
            result.error500("参数不识别！");
        } else {
            this.bytedanceAppDownloadDetailPageService.removeByIds(Arrays.asList(ids.split(StringUtils.COMMA)));
            result.success("删除成功!");
        }
        return result;
    }

    /**
     * 通过id查询
     *
     * @param id
     * @return
     */
    @AutoLog(value = "应用下载详情页信息-通过id查询")
    @ApiOperation(value = "应用下载详情页信息-通过id查询", notes = "应用下载详情页信息-通过id查询")
    @GetMapping(value = "/queryById")
    public Result<BytedanceAppDownloadDetailPage> queryById(@RequestParam(name = "id", required = true) String id) {
        Result<BytedanceAppDownloadDetailPage> result = new Result<>();
        BytedanceAppDownloadDetailPage bytedanceAppDownloadDetailPage = bytedanceAppDownloadDetailPageService.getById(id);
        if (bytedanceAppDownloadDetailPage == null) {
            result.error500("未找到对应实体");
        } else {
            result.setResult(bytedanceAppDownloadDetailPage);
            result.setSuccess(true);
        }
        return result;
    }

    /**
     * 导出excel
     *
     * @param request
     * @param response
     */
    @RequestMapping(value = "/exportXls")
    public ModelAndView exportXls(HttpServletRequest request, HttpServletResponse response) {
        // Step.1 组装查询条件
        QueryWrapper<BytedanceAppDownloadDetailPage> queryWrapper = null;
        try {
            String paramsStr = request.getParameter("paramsStr");
            if (oConvertUtils.isNotEmpty(paramsStr)) {
                String deString = URLDecoder.decode(paramsStr, "UTF-8");
                BytedanceAppDownloadDetailPage bytedanceAppDownloadDetailPage = JSON.parseObject(deString, BytedanceAppDownloadDetailPage.class);
                queryWrapper = QueryGenerator.initQueryWrapper(bytedanceAppDownloadDetailPage, request.getParameterMap());
            }
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }

        //Step.2 AutoPoi 导出Excel
        ModelAndView mv = new ModelAndView(new JeecgEntityExcelView());
        List<BytedanceAppDownloadDetailPage> pageList = bytedanceAppDownloadDetailPageService.list(queryWrapper);
        //导出文件名称
        mv.addObject(NormalExcelConstants.FILE_NAME, "应用下载详情页信息列表");
        mv.addObject(NormalExcelConstants.CLASS, BytedanceAppDownloadDetailPage.class);
        mv.addObject(NormalExcelConstants.PARAMS, new ExportParams("应用下载详情页信息列表数据", "导出人:Jeecg", "导出信息"));
        mv.addObject(NormalExcelConstants.DATA_LIST, pageList);
        return mv;
    }

    /**
     * 通过excel导入数据
     *
     * @param request
     * @param response
     * @return
     */
    @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
    public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
        MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
        Map<String, MultipartFile> fileMap = multipartRequest.getFileMap();
        for (Map.Entry<String, MultipartFile> entity : fileMap.entrySet()) {
            MultipartFile file = entity.getValue();
            ImportParams params = new ImportParams();
            params.setTitleRows(2);
            params.setHeadRows(1);
            params.setNeedSave(true);
            try {
                List<BytedanceAppDownloadDetailPage> listBytedanceAppDownloadDetailPages = ExcelImportUtil.importExcel(file.getInputStream(), BytedanceAppDownloadDetailPage.class, params);
                bytedanceAppDownloadDetailPageService.saveBatch(listBytedanceAppDownloadDetailPages);
                return Result.ok("文件导入成功！数据行数:" + listBytedanceAppDownloadDetailPages.size());
            } catch (Exception e) {
                log.error(e.getMessage(), e);
                return Result.error("文件导入失败:" + e.getMessage());
            } finally {
                try {
                    file.getInputStream().close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
        return Result.ok("文件导入失败！");
    }

}
