package cn.com.ctop.toutiao.controller;

import cn.com.ctop.common.module.utils.StringUtils;
import cn.com.ctop.toutiao.entity.BytedanceAreaInfo;
import cn.com.ctop.toutiao.service.IBytedanceAreaInfoService;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.aspect.annotation.AutoLog;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.common.util.oConvertUtils;
import org.jeecgframework.poi.excel.ExcelImportUtil;
import org.jeecgframework.poi.excel.def.NormalExcelConstants;
import org.jeecgframework.poi.excel.entity.ExportParams;
import org.jeecgframework.poi.excel.entity.ImportParams;
import org.jeecgframework.poi.excel.view.JeecgEntityExcelView;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.util.Arrays;
import java.util.List;
import java.util.Map;

/**
 * 省市县信息
 *
 * @author jeecg-boot
 * @version V1.0
 * @date 2019-09-17
 */
@Slf4j
@Api(tags = "省市县信息")
@RestController
@RequestMapping("/ctop/bytedanceAreaInfo")
public class BytedanceAreaInfoController {
    @Autowired
    private IBytedanceAreaInfoService bytedanceAreaInfoService;

    /**
     * 分页列表查询
     *
     * @param code
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    @AutoLog(value = "省市县信息-分页列表查询")
    @ApiOperation(value = "省市县信息-分页列表查询", notes = "省市县信息-分页列表查询")
    @GetMapping(value = "/listbycode")
    public Result<IPage<BytedanceAreaInfo>> queryPageListByCode(@RequestParam(name = "parentCode", defaultValue = "0") String code,
                                                                @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                                @RequestParam(name = "pageSize", defaultValue = "100") Integer pageSize,
                                                                HttpServletRequest req) {
        Result<IPage<BytedanceAreaInfo>> result = new Result<>();
        BytedanceAreaInfo areaInfo = new BytedanceAreaInfo();
        areaInfo.setParentCode(code);
        QueryWrapper<BytedanceAreaInfo> queryWrapper = QueryGenerator.initQueryWrapper(areaInfo, req.getParameterMap());
        Page<BytedanceAreaInfo> page = new Page<>(pageNo, pageSize);
        IPage<BytedanceAreaInfo> pageList = bytedanceAreaInfoService.page(page, queryWrapper);
        result.setSuccess(true);
        result.setResult(pageList);
        return result;
    }

    /**
     * 分页列表查询
     *
     * @param bytedanceAreaInfo
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    @AutoLog(value = "省市县信息-分页列表查询")
    @ApiOperation(value = "省市县信息-分页列表查询", notes = "省市县信息-分页列表查询")
    @GetMapping(value = "/list")
    public Result<IPage<BytedanceAreaInfo>> queryPageList(BytedanceAreaInfo bytedanceAreaInfo,
                                                          @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                          @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                                          HttpServletRequest req) {
        Result<IPage<BytedanceAreaInfo>> result = new Result<IPage<BytedanceAreaInfo>>();
        QueryWrapper<BytedanceAreaInfo> queryWrapper = QueryGenerator.initQueryWrapper(bytedanceAreaInfo, req.getParameterMap());
        Page<BytedanceAreaInfo> page = new Page<BytedanceAreaInfo>(pageNo, pageSize);
        IPage<BytedanceAreaInfo> pageList = bytedanceAreaInfoService.page(page, queryWrapper);
        result.setSuccess(true);
        result.setResult(pageList);
        return result;
    }

    /**
     * 添加
     *
     * @param bytedanceAreaInfo
     * @return
     */
    @AutoLog(value = "省市县信息-添加")
    @ApiOperation(value = "省市县信息-添加", notes = "省市县信息-添加")
    @PostMapping(value = "/add")
    public Result<BytedanceAreaInfo> add(@RequestBody BytedanceAreaInfo bytedanceAreaInfo) {
        Result<BytedanceAreaInfo> result = new Result<BytedanceAreaInfo>();
        try {
            bytedanceAreaInfoService.save(bytedanceAreaInfo);
            result.success("添加成功！");
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            result.error500("操作失败");
        }
        return result;
    }

    /**
     * 编辑
     *
     * @param bytedanceAreaInfo
     * @return
     */
    @AutoLog(value = "省市县信息-编辑")
    @ApiOperation(value = "省市县信息-编辑", notes = "省市县信息-编辑")
    @PutMapping(value = "/edit")
    public Result<BytedanceAreaInfo> edit(@RequestBody BytedanceAreaInfo bytedanceAreaInfo) {
        Result<BytedanceAreaInfo> result = new Result<BytedanceAreaInfo>();
        BytedanceAreaInfo bytedanceAreaInfoEntity = bytedanceAreaInfoService.getById(bytedanceAreaInfo.getId());
        if (bytedanceAreaInfoEntity == null) {
            result.error500("未找到对应实体");
        } else {
            boolean ok = bytedanceAreaInfoService.updateById(bytedanceAreaInfo);
            if (ok) {
                result.success("修改成功!");
            }
        }

        return result;
    }

    /**
     * 通过id删除
     *
     * @param id
     * @return
     */
    @AutoLog(value = "省市县信息-通过id删除")
    @ApiOperation(value = "省市县信息-通过id删除", notes = "省市县信息-通过id删除")
    @DeleteMapping(value = "/delete")
    public Result<?> delete(@RequestParam(name = "id", required = true) String id) {
        try {
            bytedanceAreaInfoService.removeById(id);
        } catch (Exception e) {
            log.error("删除失败", e.getMessage());
            return Result.error("删除失败!");
        }
        return Result.ok("删除成功!");
    }

    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @AutoLog(value = "省市县信息-批量删除")
    @ApiOperation(value = "省市县信息-批量删除", notes = "省市县信息-批量删除")
    @DeleteMapping(value = "/deleteBatch")
    public Result<BytedanceAreaInfo> deleteBatch(@RequestParam(name = "ids", required = true) String ids) {
        Result<BytedanceAreaInfo> result = new Result<BytedanceAreaInfo>();
        if (ids == null || "".equals(ids.trim())) {
            result.error500("参数不识别！");
        } else {
            this.bytedanceAreaInfoService.removeByIds(Arrays.asList(ids.split(StringUtils.COMMA)));
            result.success("删除成功!");
        }
        return result;
    }

    /**
     * 通过id查询
     *
     * @param id
     * @return
     */
    @AutoLog(value = "省市县信息-通过id查询")
    @ApiOperation(value = "省市县信息-通过id查询", notes = "省市县信息-通过id查询")
    @GetMapping(value = "/queryById")
    public Result<BytedanceAreaInfo> queryById(@RequestParam(name = "id", required = true) String id) {
        Result<BytedanceAreaInfo> result = new Result<BytedanceAreaInfo>();
        BytedanceAreaInfo bytedanceAreaInfo = bytedanceAreaInfoService.getById(id);
        if (bytedanceAreaInfo == null) {
            result.error500("未找到对应实体");
        } else {
            result.setResult(bytedanceAreaInfo);
            result.setSuccess(true);
        }
        return result;
    }

    /**
     * 导出excel
     *
     * @param request
     * @param response
     */
    @RequestMapping(value = "/exportXls")
    public ModelAndView exportXls(HttpServletRequest request, HttpServletResponse response) {
        // Step.1 组装查询条件
        QueryWrapper<BytedanceAreaInfo> queryWrapper = null;
        try {
            String paramsStr = request.getParameter("paramsStr");
            if (oConvertUtils.isNotEmpty(paramsStr)) {
                String deString = URLDecoder.decode(paramsStr, "UTF-8");
                BytedanceAreaInfo bytedanceAreaInfo = JSON.parseObject(deString, BytedanceAreaInfo.class);
                queryWrapper = QueryGenerator.initQueryWrapper(bytedanceAreaInfo, request.getParameterMap());
            }
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }

        //Step.2 AutoPoi 导出Excel
        ModelAndView mv = new ModelAndView(new JeecgEntityExcelView());
        List<BytedanceAreaInfo> pageList = bytedanceAreaInfoService.list(queryWrapper);
        //导出文件名称
        mv.addObject(NormalExcelConstants.FILE_NAME, "省市县信息列表");
        mv.addObject(NormalExcelConstants.CLASS, BytedanceAreaInfo.class);
        mv.addObject(NormalExcelConstants.PARAMS, new ExportParams("省市县信息列表数据", "导出人:Jeecg", "导出信息"));
        mv.addObject(NormalExcelConstants.DATA_LIST, pageList);
        return mv;
    }

    /**
     * 通过excel导入数据
     *
     * @param request
     * @param response
     * @return
     */
    @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
    public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
        MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
        Map<String, MultipartFile> fileMap = multipartRequest.getFileMap();
        for (Map.Entry<String, MultipartFile> entity : fileMap.entrySet()) {
            MultipartFile file = entity.getValue();
            ImportParams params = new ImportParams();
            params.setTitleRows(2);
            params.setHeadRows(1);
            params.setNeedSave(true);
            try {
                List<BytedanceAreaInfo> listBytedanceAreaInfos = ExcelImportUtil.importExcel(file.getInputStream(), BytedanceAreaInfo.class, params);
                bytedanceAreaInfoService.saveBatch(listBytedanceAreaInfos);
                return Result.ok("文件导入成功！数据行数:" + listBytedanceAreaInfos.size());
            } catch (Exception e) {
                log.error(e.getMessage(), e);
                return Result.error("文件导入失败:" + e.getMessage());
            } finally {
                try {
                    file.getInputStream().close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
        return Result.ok("文件导入失败！");
    }

}
